"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExecutorImpl = void 0;
const parallel_universe_1 = require("parallel-universe");
const utils_js_1 = require("./utils.js");
/**
 * The {@link Executor} implementation returned by the {@link ExecutorManager}.
 *
 * @internal
 */
class ExecutorImpl {
    get isRejected() {
        return this.isSettled && !this.isFulfilled;
    }
    get isSettled() {
        return this.settledAt !== 0;
    }
    get isActive() {
        return this._activationCount !== 0;
    }
    get isPending() {
        return this.promise !== null;
    }
    get isInvalidated() {
        return this.invalidatedAt !== 0;
    }
    constructor(key, manager) {
        this.key = key;
        this.manager = manager;
        this.value = undefined;
        this.reason = undefined;
        this.task = null;
        this.settledAt = 0;
        this.invalidatedAt = 0;
        this.isFulfilled = false;
        this.annotations = Object.create(null);
        this.version = 0;
        this.promise = null;
        /**
         * The number of times the executor was activated.
         */
        this._activationCount = 0;
        /**
         * The pubsub that handles the executor subscriptions.
         */
        this._pubSub = new parallel_universe_1.PubSub();
    }
    get() {
        if (this.isFulfilled) {
            return this.value;
        }
        throw this.isSettled ? this.reason : new Error('The executor is not settled');
    }
    getOrDefault(defaultValue) {
        return this.isFulfilled ? this.value : defaultValue;
    }
    getOrAwait() {
        return new parallel_universe_1.AbortablePromise((resolve, reject, signal) => {
            if (this.isSettled && !this.isPending) {
                if (this.isFulfilled) {
                    resolve(this.value);
                }
                else {
                    reject(this.reason);
                }
                return;
            }
            const unsubscribe = this.subscribe(event => {
                if (event.type === 'detached') {
                    unsubscribe();
                    reject((0, utils_js_1.AbortError)('The executor was detached'));
                    return;
                }
                if (this.isSettled && !this.isPending) {
                    unsubscribe();
                    if (this.isFulfilled) {
                        resolve(this.value);
                    }
                    else {
                        reject(this.reason);
                    }
                }
            });
            signal.addEventListener('abort', unsubscribe);
        });
    }
    execute(task) {
        const promise = new parallel_universe_1.AbortablePromise((resolve, reject, signal) => {
            signal.addEventListener('abort', () => {
                if (this.promise === promise) {
                    this.promise = null;
                    this.version++;
                }
                this.publish({ type: 'aborted' });
            });
            new Promise(resolve => {
                const value = task(signal, this);
                resolve(value instanceof parallel_universe_1.AbortablePromise ? value.withSignal(signal) : value);
            }).then(value => {
                if (signal.aborted) {
                    return;
                }
                this.promise = null;
                this.resolve(value);
                resolve(value);
            }, reason => {
                if (signal.aborted) {
                    return;
                }
                this.promise = null;
                this.reject(reason);
                reject(reason);
            });
        });
        promise.catch(utils_js_1.noop);
        const prevPromise = this.promise;
        this.promise = promise;
        if (prevPromise !== null) {
            prevPromise.abort((0, utils_js_1.AbortError)('The task was replaced'));
        }
        else {
            this.version++;
        }
        if (this.promise === promise) {
            this.task = task;
            this.publish({ type: 'pending' });
        }
        return promise;
    }
    retry() {
        if (this.task !== null && !this.isPending) {
            this.execute(this.task);
        }
    }
    clear() {
        if (this.isSettled) {
            this.isFulfilled = false;
            this.value = this.reason = undefined;
            this.settledAt = this.invalidatedAt = 0;
            this.version++;
            this.publish({ type: 'cleared' });
        }
    }
    abort(reason = (0, utils_js_1.AbortError)('The executor was aborted')) {
        var _a;
        (_a = this.promise) === null || _a === void 0 ? void 0 : _a.abort(reason);
    }
    invalidate(invalidatedAt = Date.now()) {
        if (!this.isInvalidated && this.isSettled) {
            this.invalidatedAt = invalidatedAt;
            this.version++;
            this.publish({ type: 'invalidated' });
        }
    }
    resolve(value, settledAt = Date.now()) {
        if ((0, utils_js_1.isObjectLike)(value) && 'then' in value) {
            this.execute(() => value);
            return;
        }
        const prevPromise = this.promise;
        this.promise = null;
        prevPromise === null || prevPromise === void 0 ? void 0 : prevPromise.abort();
        this.isFulfilled = true;
        this.value = value;
        this.settledAt = settledAt;
        this.invalidatedAt = 0;
        this.version++;
        this.publish({ type: 'fulfilled' });
    }
    reject(reason, settledAt = Date.now()) {
        const prevPromise = this.promise;
        this.promise = null;
        prevPromise === null || prevPromise === void 0 ? void 0 : prevPromise.abort();
        this.isFulfilled = false;
        this.reason = reason;
        this.settledAt = settledAt;
        this.invalidatedAt = 0;
        this.version++;
        this.publish({ type: 'rejected' });
    }
    activate() {
        let isApplicable = true;
        if (this._activationCount++ === 0) {
            this.publish({ type: 'activated' });
        }
        return () => {
            if (isApplicable && ((isApplicable = false), --this._activationCount === 0)) {
                this.publish({ type: 'deactivated' });
            }
        };
    }
    subscribe(listener) {
        return this._pubSub.subscribe(listener);
    }
    publish(event) {
        this._pubSub.publish({
            type: event.type,
            target: this,
            version: this.version,
            payload: event.payload,
        });
    }
    annotate(patch) {
        this.version++;
        Object.assign(this.annotations, patch);
        this.publish({ type: 'annotated' });
    }
    toJSON() {
        return {
            key: this.key,
            isFulfilled: this.isFulfilled,
            value: this.value,
            reason: this.reason,
            annotations: this.annotations,
            settledAt: this.settledAt,
            invalidatedAt: this.invalidatedAt,
        };
    }
}
exports.ExecutorImpl = ExecutorImpl;
