import { AbortablePromise, PubSub } from 'parallel-universe';
import { ExecutorImpl } from './ExecutorImpl.js';
import type { Executor, ExecutorEvent, ExecutorPlugin, ExecutorState, ExecutorTask, NoInfer, Observable } from './types.js';
/**
 * Options provided to the {@link ExecutorManager} constructor.
 */
export interface ExecutorManagerOptions {
    /**
     * Plugins that are applied to all executors.
     */
    plugins?: Array<ExecutorPlugin | null | undefined>;
    /**
     * Serializes executor keys.
     *
     * The serialized key form can be anything. If you want to use object identities as executor keys, provide an identity
     * function as a serializer.
     *
     * @param key The key to serialize.
     * @default JSON.stringify
     */
    keySerializer?: (key: any) => any;
    /**
     * If `true` then executors are registered in the devtools extension.
     *
     * @default true
     */
    devtools?: boolean;
}
/**
 * Creates executors and manages their lifecycle.
 */
export declare class ExecutorManager implements Iterable<Executor>, Observable<ExecutorEvent> {
    /**
     * The map from a key to an executor.
     */
    protected readonly _executors: Map<unknown, ExecutorImpl<any>>;
    /**
     * The pubsub that handles the manager subscriptions.
     */
    protected readonly _pubSub: PubSub<ExecutorEvent<any>>;
    /**
     * The map from a key to an initial state that must be set to an executor before plugins are applied. Entries from
     * this map are deleted after the executor is initialized.
     */
    protected readonly _initialState: Map<unknown, ExecutorState<any>>;
    /**
     * Plugins that are applied to all executors.
     */
    protected readonly _plugins: ExecutorPlugin[];
    /**
     * Serializes keys of executors.
     */
    readonly keySerializer: (key: unknown) => unknown;
    /**
     * Creates a new executor manager.
     *
     * @param options Additional options.
     */
    constructor(options?: ExecutorManagerOptions);
    /**
     * Returns an executor by its key, or `undefined` if there's no such executor.
     *
     * @param key The unique executor key.
     */
    get(key: unknown): Executor | undefined;
    /**
     * Returns an existing executor or creates a new one.
     *
     * @param key The unique executor key.
     * @param initialValue The initial executor value.
     * @param plugins The array of plugins that are applied to the newly created executor.
     */
    getOrCreate<Value = any>(key: unknown, initialValue: undefined, plugins?: Array<ExecutorPlugin<Value> | null | undefined>): Executor<Value>;
    /**
     * Returns an existing executor or creates a new one.
     *
     * @param key The unique executor key.
     * @param initialValue The initial executor value.
     * @param plugins The array of plugins that are applied to the newly created executor.
     */
    getOrCreate<Value = any>(key: unknown, initialValue?: ExecutorTask<Value> | PromiseLike<Value> | Value, plugins?: Array<ExecutorPlugin<NoInfer<Value>> | null | undefined>): Executor<Value>;
    /**
     * Resolves with the existing executor or waits for an executor to be created.
     *
     * @param key The executor key to wait for.
     */
    getOrAwait(key: unknown): AbortablePromise<Executor>;
    /**
     * Deletes the non-{@link Executor.isActive active} executor from the manager.
     *
     * If the detached executor is {@link Executor.isPending pending} then it _is not_ aborted. Subscribe to the
     * {@link ExecutorEvent detach} event on either manager or an executor and abort it manually.
     *
     * @param key The key of the executor to delete.
     * @returns `true` if the executor was detached, or `false` if there's no such executor, or the executor is active.
     */
    detach(key: unknown): boolean;
    /**
     * Subscribes a listener to the events published by all executors that are created by this manager.
     *
     * @param listener The listener to subscribe.
     * @returns The callback that unsubscribes the listener.
     */
    subscribe(listener: (event: ExecutorEvent) => void): () => void;
    /**
     * Iterates over executors created by the manager.
     */
    [Symbol.iterator](): IterableIterator<Executor>;
    /**
     * Returns serializable executor manager state.
     */
    toJSON(): ExecutorState[];
    /**
     * Injects the initial state for the executor that is created in the future.
     *
     * @param initialState The initial state of the hydrated executor that can be created via
     * {@link ExecutorManager.getOrCreate}.
     * @returns `true` if the executor was hydrated, or `false` if the executor already exists and cannot be hydrated.
     */
    hydrate(initialState: ExecutorState): boolean;
}
