import { AbortablePromise, PubSub } from 'parallel-universe';
import { ExecutorImpl } from './ExecutorImpl.mjs';
/**
 * Creates executors and manages their lifecycle.
 */
export class ExecutorManager {
    /**
     * Creates a new executor manager.
     *
     * @param options Additional options.
     */
    constructor(options = {}) {
        /**
         * The map from a key to an executor.
         */
        this._executors = new Map();
        /**
         * The pubsub that handles the manager subscriptions.
         */
        this._pubSub = new PubSub();
        /**
         * The map from a key to an initial state that must be set to an executor before plugins are applied. Entries from
         * this map are deleted after the executor is initialized.
         */
        this._initialState = new Map();
        /**
         * Plugins that are applied to all executors.
         */
        this._plugins = [];
        this.keySerializer = options.keySerializer || (value => JSON.stringify(value));
        if (options.devtools === undefined || options.devtools) {
            const devtools = typeof __REACT_EXECUTOR_DEVTOOLS__ !== 'undefined' ? __REACT_EXECUTOR_DEVTOOLS__ : undefined;
            if (devtools !== undefined) {
                this._plugins.push(devtools.plugin);
            }
        }
        if (options.plugins !== undefined) {
            for (const plugin of options.plugins) {
                if (plugin !== null && plugin !== undefined) {
                    this._plugins.push(plugin);
                }
            }
        }
    }
    /**
     * Returns an executor by its key, or `undefined` if there's no such executor.
     *
     * @param key The unique executor key.
     */
    get(key) {
        return this._executors.get(this.keySerializer(key));
    }
    getOrCreate(key, initialValue, plugins) {
        const serializedKey = this.keySerializer(key);
        let executor = this._executors.get(serializedKey);
        if (executor !== undefined) {
            // Existing executor
            return executor;
        }
        executor = Object.assign(new ExecutorImpl(key, this), this._initialState.get(serializedKey));
        if (typeof initialValue === 'function') {
            executor.task = initialValue;
        }
        const unsubscribe = executor.subscribe(event => {
            this._pubSub.publish(event);
        });
        try {
            for (const plugin of this._plugins) {
                plugin(executor);
            }
            if (plugins !== undefined) {
                for (const plugin of plugins) {
                    plugin === null || plugin === void 0 ? void 0 : plugin(executor);
                }
            }
        }
        catch (error) {
            unsubscribe();
            throw error;
        }
        this._initialState.delete(serializedKey);
        this._executors.set(serializedKey, executor);
        executor.publish({ type: 'attached' });
        if (initialValue === undefined || executor.isSettled || executor.isPending) {
            return executor;
        }
        if (typeof initialValue === 'function') {
            executor.execute(initialValue);
        }
        else {
            executor.resolve(initialValue);
        }
        return executor;
    }
    /**
     * Resolves with the existing executor or waits for an executor to be created.
     *
     * @param key The executor key to wait for.
     */
    getOrAwait(key) {
        return new AbortablePromise((resolve, _reject, signal) => {
            const serializedKey = this.keySerializer(key);
            const executor = this._executors.get(serializedKey);
            if (executor !== undefined) {
                resolve(executor);
                return;
            }
            const unsubscribe = this.subscribe(event => {
                if (event.type === 'attached' && this.keySerializer(event.target.key) === serializedKey) {
                    unsubscribe();
                    resolve(event.target);
                }
            });
            signal.addEventListener('abort', unsubscribe);
        });
    }
    /**
     * Deletes the non-{@link Executor.isActive active} executor from the manager.
     *
     * If the detached executor is {@link Executor.isPending pending} then it _is not_ aborted. Subscribe to the
     * {@link ExecutorEvent detach} event on either manager or an executor and abort it manually.
     *
     * @param key The key of the executor to delete.
     * @returns `true` if the executor was detached, or `false` if there's no such executor, or the executor is active.
     */
    detach(key) {
        const serializedKey = this.keySerializer(key);
        const executor = this._executors.get(serializedKey);
        if (executor === undefined || executor.isActive) {
            return false;
        }
        this._executors.delete(serializedKey);
        executor.publish({ type: 'detached' });
        executor._pubSub.unsubscribeAll();
        return true;
    }
    /**
     * Subscribes a listener to the events published by all executors that are created by this manager.
     *
     * @param listener The listener to subscribe.
     * @returns The callback that unsubscribes the listener.
     */
    subscribe(listener) {
        return this._pubSub.subscribe(listener);
    }
    /**
     * Iterates over executors created by the manager.
     */
    [Symbol.iterator]() {
        return this._executors.values();
    }
    /**
     * Returns serializable executor manager state.
     */
    toJSON() {
        return Array.from(this).map(executor => executor.toJSON());
    }
    /**
     * Injects the initial state for the executor that is created in the future.
     *
     * @param initialState The initial state of the hydrated executor that can be created via
     * {@link ExecutorManager.getOrCreate}.
     * @returns `true` if the executor was hydrated, or `false` if the executor already exists and cannot be hydrated.
     */
    hydrate(initialState) {
        const serializedKey = this.keySerializer(initialState.key);
        if (this._executors.has(serializedKey)) {
            return false;
        }
        this._initialState.set(serializedKey, initialState);
        return true;
    }
}
