"use strict";
/**
 * The observable that emits `true` if
 * [the device is _connected_ to the network](https://developer.mozilla.org/en-US/docs/Web/API/Navigator/onLine), and
 * emits `false` if the device is disconnected from the network.
 *
 * ```ts
 * import retryWhen from 'react-executor/plugin/retryWhen';
 * import navigatorOnline from 'react-executor/observable/navigatorOnline';
 *
 * const executor = useExecutor('test', heavyTask, [
 *   retryWhen(navigatorOnline)
 * ]);
 * ```
 *
 * @module observable/navigatorOnline
 */
Object.defineProperty(exports, "__esModule", { value: true });
const parallel_universe_1 = require("parallel-universe");
const utils_js_1 = require("../utils.js");
const pubSub = new parallel_universe_1.PubSub();
function handleChange() {
    pubSub.publish(window.navigator.onLine);
}
/**
 * The observable that emits `true` if
 * [the device is _connected_ to the network](https://developer.mozilla.org/en-US/docs/Web/API/Navigator/onLine), and
 * emits `false` if the device is disconnected from the network.
 */
const navigatorOnline = {
    subscribe(listener) {
        if (typeof window === 'undefined') {
            return utils_js_1.noop;
        }
        if (pubSub.listenerCount === 0) {
            window.addEventListener('offline', handleChange, false);
            window.addEventListener('online', handleChange, false);
        }
        const unsubscribe = pubSub.subscribe(listener);
        setTimeout(handleChange, 0);
        return () => {
            unsubscribe();
            if (pubSub.listenerCount === 0) {
                window.removeEventListener('offline', handleChange, false);
                window.removeEventListener('online', handleChange, false);
            }
        };
    },
};
exports.default = navigatorOnline;
