/**
 * The plugin that detaches an executor that isn't active after a delay.
 *
 * ```ts
 * import detachInactive from 'react-executor/plugin/detachInactive';
 *
 * const executor = useExecutor('test', 42, [
 *   detachInactive({ delayBeforeActivation: 5_000 })
 * ]);
 * ```
 *
 * @module plugin/detachInactive
 */
/**
 * Detaches an executor that isn't active after a delay.
 *
 * @param options Detach options.
 */
export default function detachInactive(options) {
    const { delayBeforeActivation, delayAfterDeactivation } = options;
    return executor => {
        let timer;
        if (delayBeforeActivation !== undefined) {
            timer = setTimeout(() => executor.manager.detach(executor.key), delayBeforeActivation);
        }
        executor.subscribe(event => {
            switch (event.type) {
                case 'deactivated':
                    clearTimeout(timer);
                    if (delayAfterDeactivation !== undefined) {
                        timer = setTimeout(() => executor.manager.detach(executor.key), delayAfterDeactivation);
                    }
                    break;
                case 'activated':
                case 'detached':
                    clearTimeout(timer);
                    break;
            }
        });
        executor.publish({
            type: 'plugin_configured',
            payload: {
                type: 'detachInactive',
                options: { delayBeforeActivation, delayAfterDeactivation },
            },
        });
    };
}
