"use strict";
/**
 * The plugin that invalidates the executor result if another executor is fulfilled or invalidated.
 *
 * ```ts
 * import invalidateByPeers from 'react-executor/plugin/invalidateByPeers';
 *
 * const executor = useExecutor('test', 42, [
 *   invalidateByPeers(executor => executor.key === 'exact_executor_key')
 * ]);
 * ```
 *
 * @module plugin/invalidateByPeers
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = invalidateByPeers;
/**
 * Invalidates the executor result if another executor is fulfilled or invalidated.
 *
 * @param peerMatcher A collection of peer executors or a callback that returns a truthy value for a matching peer
 * executor.
 */
function invalidateByPeers(peerMatcher) {
    return executor => {
        const peerExecutors = new Set(typeof peerMatcher === 'function' ? Array.from(executor.manager).filter(peerMatcher) : peerMatcher);
        const unsubscribe = executor.manager.subscribe(event => {
            const { type: eventType, target: peerExecutor } = event;
            if (peerExecutor === executor) {
                if (eventType === 'detached') {
                    unsubscribe();
                }
                return;
            }
            if ((eventType === 'attached' &&
                typeof peerMatcher === 'function' &&
                peerMatcher(peerExecutor) &&
                (peerExecutors.add(peerExecutor), true)) ||
                (eventType === 'detached' && peerExecutors.delete(peerExecutor))) {
                executor.publish({
                    type: 'plugin_configured',
                    payload: {
                        type: 'invalidateByPeers',
                        options: { peerExecutors: Array.from(peerExecutors) },
                    },
                });
            }
            if ((eventType === 'invalidated' || eventType === 'fulfilled') && peerExecutors.has(peerExecutor)) {
                executor.invalidate();
            }
        });
        executor.publish({
            type: 'plugin_configured',
            payload: {
                type: 'invalidateByPeers',
                options: { peerExecutors: Array.from(peerExecutors) },
            },
        });
    };
}
