"use strict";
/**
 * The plugin that aborts the pending task and rejects the executor with {@link !DOMException TimeoutError} if the task
 * execution took longer then the delay.
 *
 * ```ts
 * import rejectPendingAfter from 'react-executor/plugin/rejectPendingAfter';
 *
 * const executor = useExecutor('test', heavyTask, [
 *   rejectPendingAfter(10_000)
 * ]);
 * ```
 *
 * @module plugin/rejectPendingAfter
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = rejectPendingAfter;
const utils_js_1 = require("../utils.js");
/**
 * Aborts the pending task and rejects the executor with {@link !DOMException TimeoutError} if the task execution took
 * longer then the delay.
 *
 * @param delay The delay in milliseconds after which the executor is rejected.
 */
function rejectPendingAfter(delay) {
    return executor => {
        let timer;
        executor.subscribe(event => {
            switch (event.type) {
                case 'pending':
                    timer = setTimeout(() => {
                        const error = (0, utils_js_1.TimeoutError)('The task execution took too long');
                        executor.abort(error);
                        executor.reject(error);
                    }, delay);
                    break;
                case 'fulfilled':
                case 'rejected':
                case 'aborted':
                case 'detached':
                    clearTimeout(timer);
                    break;
            }
        });
        executor.publish({
            type: 'plugin_configured',
            payload: {
                type: 'rejectPendingAfter',
                options: { delay },
            },
        });
    };
}
