/**
 * The plugin that retries the last task if it has is failed.
 *
 * ```ts
 * import retryRejected from 'react-executor/plugin/retryRejected';
 *
 * const executor = useExecutor('test', heavyTask, [retryRejected()]);
 * ```
 *
 * @module plugin/retryRejected
 */
import { emptyObject } from '../utils.mjs';
/**
 * Retries the last task if it has is failed.
 *
 * @param options Retry options.
 * @template Value The value stored by the executor.
 */
export default function retryRejected(options = emptyObject) {
    const { count = 3, delay = exponentialBackoff, isEager = false } = options;
    return executor => {
        let timer;
        let index = 0;
        executor.subscribe(event => {
            switch (event.type) {
                case 'activated':
                case 'rejected':
                    clearTimeout(timer);
                    if (!executor.isPending && (isEager || executor.isActive) && executor.isRejected && index < count) {
                        timer = setTimeout(() => {
                            index++;
                            executor.retry();
                        }, (typeof delay === 'function' ? delay(index, executor) : delay) - Date.now() + executor.settledAt);
                    }
                    break;
                case 'fulfilled':
                case 'aborted':
                case 'deactivated':
                case 'detached':
                    index = 0;
                    clearTimeout(timer);
                    break;
            }
        });
        executor.publish({
            type: 'plugin_configured',
            payload: {
                type: 'retryRejected',
                options: { count, delay, isEager },
            },
        });
    };
}
function exponentialBackoff(index) {
    return 1000 * Math.pow(1.8, index);
}
