/**
 * The plugin that retries the latest task if the observable emits `true`.
 *
 * ```ts
 * import retryWhen from 'react-executor/plugin/retryWhen';
 * import windowFocused from 'react-executor/observable/windowFocused';
 *
 * const executor = useExecutor('test', heavyTask, [
 *   retryWhen(windowFocused)
 * ]);
 * ```
 *
 * @module plugin/retryWhen
 */
import type { ExecutorPlugin, Observable } from '../types.js';
/**
 * Options of the {@link retryWhen} plugin.
 */
export interface RetryWhenOptions {
    /**
     * The delay in milliseconds after `true` is emitted by the observer and before the executor is retried. If during
     * this delay `false` is emitted, then executor isn't retried.
     *
     * @default 0
     */
    delay?: number;
    /**
     * If `true` then executor is retried even if it isn't active.
     *
     * @default false
     */
    isEager?: boolean;
}
/**
 * Retries the latest task if the observable emits `true`. If executor isn't active and retry isn't
 * {@link RetryWhenOptions.isEager eager} then the task is retried after the executor becomes active.
 *
 * @param observable The observable that triggers the retry of the latest task.
 * @param options Retry options.
 */
export default function retryWhen(observable: Observable<boolean>, options?: RetryWhenOptions): ExecutorPlugin;
