"use strict";
/**
 * The plugin that retries the latest task if the observable emits `true`.
 *
 * ```ts
 * import retryWhen from 'react-executor/plugin/retryWhen';
 * import windowFocused from 'react-executor/observable/windowFocused';
 *
 * const executor = useExecutor('test', heavyTask, [
 *   retryWhen(windowFocused)
 * ]);
 * ```
 *
 * @module plugin/retryWhen
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = retryWhen;
const utils_js_1 = require("../utils.js");
/**
 * Retries the latest task if the observable emits `true`. If executor isn't active and retry isn't
 * {@link RetryWhenOptions.isEager eager} then the task is retried after the executor becomes active.
 *
 * @param observable The observable that triggers the retry of the latest task.
 * @param options Retry options.
 */
function retryWhen(observable, options = utils_js_1.emptyObject) {
    const { delay = 0, isEager = false } = options;
    return executor => {
        let timer;
        let shouldRetry = false;
        const unsubscribe = observable.subscribe(isRetried => {
            if (!isRetried) {
                clearTimeout(timer);
                timer = undefined;
                shouldRetry = false;
                return;
            }
            if (shouldRetry || timer !== undefined || executor.isPending) {
                return;
            }
            timer = setTimeout(() => {
                if (isEager || executor.isActive) {
                    shouldRetry = false;
                    executor.retry();
                }
                else {
                    timer = undefined;
                    shouldRetry = true;
                }
            }, delay);
        });
        executor.subscribe(event => {
            switch (event.type) {
                case 'activated':
                    if (shouldRetry) {
                        shouldRetry = false;
                        executor.retry();
                    }
                    break;
                case 'pending':
                case 'fulfilled':
                case 'rejected':
                    clearTimeout(timer);
                    timer = undefined;
                    shouldRetry = false;
                    break;
                case 'detached':
                    clearTimeout(timer);
                    unsubscribe();
                    break;
            }
        });
        executor.publish({
            type: 'plugin_configured',
            payload: {
                type: 'retryWhen',
                options: { observable, delay, isEager },
            },
        });
    };
}
