/**
 * The plugin that persists the executor value in the synchronous storage.
 *
 * ```ts
 * import synchronizeStorage from 'react-executor/plugin/synchronizeStorage';
 *
 * const executor = useExecutor('test', 42, [
 *   synchronizeStorage(localStorage)
 * ]);
 * ```
 *
 * @module plugin/synchronizeStorage
 */
import type { Executor, ExecutorPlugin, ExecutorState } from '../types.js';
/**
 * Serializes and deserializes values.
 *
 * @template Value The value to serialize.
 */
export interface Serializer<Value> {
    /**
     * Serializes a value as a string.
     *
     * @param value The value to serialize.
     */
    stringify(value: Value): string;
    /**
     * Deserializes a stringified value.
     *
     * @param valueStr The stringified value.
     */
    parse(valueStr: string): Value;
}
/**
 * Options of the {@link synchronizeStorage} plugin.
 */
export interface SynchronizeStorageOptions<Value> {
    /**
     * The storage record serializer.
     */
    serializer?: Serializer<ExecutorState<Value>>;
    /**
     * A storage key, or a callback that returns the storage key.
     *
     * By default, a serialized {@link Executor.key} is used as a storage key.
     */
    storageKey?: string | ((executor: Executor) => string);
}
/**
 * Persists the executor value in the synchronous storage.
 *
 * Synchronization is enabled only for activated executors. If executor is detached, then the corresponding item is
 * removed from the storage.
 *
 * @param storage The storage where executor value is persisted, usually a `localStorage` or a `sessionStorage`.
 * @param options Additional options.
 * @template Value The value persisted in the storage.
 */
export default function synchronizeStorage<Value = any>(storage: Pick<Storage, 'setItem' | 'getItem' | 'removeItem'>, options?: SynchronizeStorageOptions<Value>): ExecutorPlugin<Value>;
