import { ExecutorManager, type ExecutorManagerOptions } from '../ExecutorManager.js';
import type { Executor, ExecutorState } from '../types.js';
/**
 * Options provided to the {@link SSRExecutorManager} constructor.
 */
export interface SSRExecutorManagerOptions extends ExecutorManagerOptions {
    /**
     * Stringifies an executor state before it is sent to the client.
     *
     * @param state The executor state to stringify.
     * @default JSON.stringify
     */
    stateStringifier?: (state: ExecutorState) => string;
    /**
     * Filters executors that must be hydrated on the client with the state that was accumulated during SSR.
     *
     * By default, only executors that were fulfilled during SSR are hydrated on the client.
     *
     * @param executor The executor to check.
     * @returns `true` if the executor must be hydrated on the client, or `false` otherwise.
     */
    executorFilter?: (executor: Executor) => boolean;
    /**
     * A nonce string to allow hydration scripts under a
     * [`script-src` Content-Security-Policy](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy/script-src).
     */
    nonce?: string;
}
/**
 * The base implementation of the executor manager that supports client hydration after SSR.
 */
export declare class SSRExecutorManager extends ExecutorManager {
    /**
     * Map from an executor to a version of the state that was sent to the client for hydration.
     */
    protected _hydratedVersions: WeakMap<Executor<any>, number>;
    /**
     * Stringifies the state of the executor before sending it to the client.
     */
    protected _stateStringifier: {
        (value: any, replacer?: (this: any, key: string, value: any) => any, space?: string | number): string;
        (value: any, replacer?: (number | string)[] | null, space?: string | number): string;
    };
    /**
     * Filters executors that must be hydrated on the client with the state that was accumulated during SSR.
     */
    protected _executorFilter: (executor: Executor) => boolean;
    /**
     * A nonce string to allow hydration scripts under a
     * [`script-src` Content-Security-Policy](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy/script-src).
     */
    nonce: string | undefined;
    /**
     * Creates a new {@link SSRExecutorManager} instance.
     *
     * @param options Additional options.
     */
    constructor(options?: SSRExecutorManagerOptions);
    /**
     * Returns an inline `<script>` tag with source that hydrates the client with the state accumulated during SSR,
     * or an empty string if there are no state changes since the last time {@link nextHydrationScript} was called.
     */
    nextHydrationChunk(): string;
    /**
     * Returns a script source that hydrates the client with the state accumulated during SSR, or an empty string if there
     * are no state changes since the last time {@link nextHydrationScript} was called.
     */
    nextHydrationScript(): string;
    /**
     * Instantly aborts all pending executors and preserves their available results as is.
     *
     * @param reason The abort reason that is used for rejection of the pending task promises.
     */
    abort(reason?: unknown): void;
    /**
     * Resolves with `true` if there were pending executors and their state has changed after they became non-pending.
     * Otherwise, resolves with `false`.
     */
    hasChanges(): Promise<boolean>;
}
