"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SSRExecutorManager = void 0;
const ExecutorManager_js_1 = require("../ExecutorManager.js");
/**
 * The base implementation of the executor manager that supports client hydration after SSR.
 */
class SSRExecutorManager extends ExecutorManager_js_1.ExecutorManager {
    /**
     * Creates a new {@link SSRExecutorManager} instance.
     *
     * @param options Additional options.
     */
    constructor(options = {}) {
        const { stateStringifier = JSON.stringify, executorFilter = executor => executor.isFulfilled } = options;
        super(options);
        /**
         * Map from an executor to a version of the state that was sent to the client for hydration.
         */
        this._hydratedVersions = new WeakMap();
        this._stateStringifier = stateStringifier;
        this._executorFilter = executorFilter;
        this.nonce = options.nonce;
    }
    /**
     * Returns an inline `<script>` tag with source that hydrates the client with the state accumulated during SSR,
     * or an empty string if there are no state changes since the last time {@link nextHydrationScript} was called.
     */
    nextHydrationChunk() {
        const source = this.nextHydrationScript();
        if (source === '') {
            return source;
        }
        return (this.nonce === undefined ? '<script>' : '<script nonce="' + this.nonce + '">') + source + '</script>';
    }
    /**
     * Returns a script source that hydrates the client with the state accumulated during SSR, or an empty string if there
     * are no state changes since the last time {@link nextHydrationScript} was called.
     */
    nextHydrationScript() {
        const stateStrs = [];
        for (const executor of this._executors.values()) {
            if (this._hydratedVersions.get(executor) !== executor.version && this._executorFilter(executor)) {
                stateStrs.push(JSON.stringify(this._stateStringifier(executor.toJSON())));
                this._hydratedVersions.set(executor, executor.version);
            }
        }
        if (stateStrs.length === 0) {
            return '';
        }
        return ('(window.__REACT_EXECUTOR_SSR_STATE__=window.__REACT_EXECUTOR_SSR_STATE__||[]).push(' +
            stateStrs.join(',').replace(/</g, '\\u003C') +
            ');var e=document.currentScript;e&&e.parentNode.removeChild(e);');
    }
    /**
     * Instantly aborts all pending executors and preserves their available results as is.
     *
     * @param reason The abort reason that is used for rejection of the pending task promises.
     */
    abort(reason) {
        for (const executor of this) {
            executor.abort(reason);
        }
    }
    /**
     * Resolves with `true` if there were pending executors and their state has changed after they became non-pending.
     * Otherwise, resolves with `false`.
     */
    hasChanges() {
        const getVersion = () => Array.from(this).reduce((version, executor) => version + executor.version, 0);
        const initialVersion = getVersion();
        const hasChanges = () => Promise.allSettled(Array.from(this).map(executor => executor.promise)).then(() => Array.from(this).some(executor => executor.isPending) ? hasChanges() : getVersion() !== initialVersion);
        return hasChanges();
    }
}
exports.SSRExecutorManager = SSRExecutorManager;
